/**
 * backend.c
 *
 * part of eeetray
 * written by SvOlli
 * distributed under the terms of the GPL
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>

#include "backend.h"

#define SUDO "/usr/bin/sudo"
#define WLAN "/etc/acpi/wlan.sh"
#define BLUE "/etc/acpi/bt.sh"
#define SSHD "/etc/init.d/ssh"

static char *const ssh_indicator = "/var/run/sshd.pid";
static char *const ssh_start[] = { SUDO, SSHD, "start", 0 };
static char *const ssh_stop[]  = { SUDO, SSHD, "stop",  0 };

static char *const bluetooth_indicator = "/proc/acpi/asus/bluetooth";
static char *const bluetooth_on[]  = { SUDO, BLUE, "poweron",    0 };
static char *const bluetooth_off[] = { SUDO, BLUE, "poweroff",   0 };

static char *const wlan_indicator = "/proc/acpi/asus/wlan";
static char *const wlan_on[]   = { SUDO, WLAN, "poweron",    0 };
static char *const wlan_off[]  = { SUDO, WLAN, "poweroff",   0 };
static char *const wlan_up[]   = { SUDO, WLAN, "resume",     0 };
static char *const wlan_down[] = { SUDO, WLAN, "disconnect_savetemp", 0 };

static char *const webcam_indicator = "/proc/acpi/asus/camera";
static char *const cardreader_indicator = "/proc/acpi/asus/cardr";
static char *const threeg_indicator = "/proc/acpi/asus/3g";

static char *const reboot[]  = { SUDO, "/sbin/fastreboot.sh",  0 };
static char *const poweroff[] = { SUDO, "/sbin/fastshutdown.sh", 0 };


void init_backend()
{
   if( fork() )
   {
      exit(0);
   }
}

int test_ssh()
{
   int success = 0;
   int fd = open( SSHD, O_RDONLY );
   if( fd < 0 )
   {
      return fd;
   }
   close( fd );
   
   fd = open( ssh_indicator, O_RDONLY );
   if( fd >= 0 )
   {
      success = 1;
      close( fd );
   }
   return success;
}

static int test_acpi( const char *filename )
{
   int success = 0;
   char status = 0;
   int fd = open( filename, O_RDONLY );
   if( fd >= 0 )
   {
      if( read( fd, &status, 1 ) > 0 )
      {
         if( status == '1' )
         {
            success = 1;
         }
      }
      close( fd );
      return success;
   }
   else
   {
      return fd;
   }
}

static int turn_acpi( const char *filename, int on )
{
   int success = 0;
   char status = on ? '1' : '0';
   int fd = open( filename, O_WRONLY );
   if( fd >= 0 )
   {
      if( write( fd, &status, 1 ) > 0 )
      {
         success = 1;
      }
      close( fd );
   }
   return success;
}


int test_wlan()
{
   return test_acpi( wlan_indicator );
}

int test_bluetooth()
{
   return test_acpi( bluetooth_indicator );
}

int test_webcam()
{
   return test_acpi( webcam_indicator );
}

int test_cardreader()
{
   return test_acpi( cardreader_indicator );
}

int test_threeg()
{
   return test_acpi( threeg_indicator );
}

static int call( char *const argv[] )
{
   pid_t cpid, w;
   int status;

   cpid = fork();
   if (cpid == -1)
   {
      perror( "fork" );
      exit(EXIT_FAILURE);
   }

   if( cpid )
   {
      /* Code executed by parent */
      w = waitpid( cpid, &status, WUNTRACED );
      if (w == -1)
      {
         perror( "waitpid" );
         exit( 1 );
      }
   }
   else
   {
      /* Code executed by child */
      /* close stdout and stderr and reopen them to redirect
         output to /dev/null */
      close( 2 );
      close( 1 );
      open( "/dev/null", O_WRONLY );
      open( "/dev/null", O_WRONLY );
      execv( argv[0], argv );
   }
   return WEXITSTATUS(status);
}

int turn_ssh( int on )
{
   if( on )
   {
      return call( ssh_start );
   }
   else
   {
      return call( ssh_stop );
   }
}

int turn_bluetooth( int on )
{
   if( on )
   {
      return call( bluetooth_on );
   }
   else
   {
      return call( bluetooth_off );
   }
}

int turn_wlan( int on )
{
   int retval;
   
   if( on )
   {
      retval = call( wlan_on );
      if( retval ) return retval;
      return call( wlan_up );
   }
   else
   {
      retval = call( wlan_down );
      return retval | call( wlan_off );
   }
}

int turn_webcam( int on )
{
   return turn_acpi( webcam_indicator, on );
}

int turn_cardreader( int on )
{
   return turn_acpi( cardreader_indicator, on );
}

int turn_threeg( int on )
{
   return turn_acpi( threeg_indicator, on );
}

int run_reboot()
{
   return call( reboot );
}

int run_poweroff()
{
   return call( poweroff );
}
